package Business::Fixflo::Version;

=head1 NAME

Business::Fixflo::Version

=head1 DESCRIPTION

A role for a fixflo version information.

=cut

use Moo::Role;

$Business::Fixflo::VERSION     = '0.20';
$Business::Fixflo::API_VERSION = 'v2';

=head1 AUTHOR

Lee Johnson - C<leejo@cpan.org>

This library is free software; you can redistribute it and/or modify it under
the same terms as Perl itself. If you would like to contribute documentation,
features, bug fixes, or anything else then please raise an issue / pull request:

    https://github.com/G3S/business-fixflo

=cut

1;
